from dataclasses import dataclass

from .extended_dataclasses import basic_dataclass, featured_dataclass


@dataclass
class MirobotAngles(featured_dataclass):
    """ A dataclass to hold Mirobot's angular values. """
    a: float = None
    """ Angle of axis 1 """
    b: float = None
    """ Angle of axis 2 """
    c: float = None
    """ Angle of axis 3 """
    x: float = None
    """ Angle of axis 4 """
    y: float = None
    """ Angle of axis 5 """
    z: float = None
    """ Angle of axis 6 """
    d: float = None
    """ Location of external slide rail module """

    @property
    def a1(self):
        """ Angle of axis 1 """
        return self.a

    @property
    def a2(self):
        """ Angle of axis 2 """
        return self.b

    @property
    def a3(self):
        """ Angle of axis 3 """
        return self.c

    @property
    def a4(self):
        """ Angle of axis 4 """
        return self.x

    @property
    def a5(self):
        """ Angle of axis 5 """
        return self.y

    @property
    def a6(self):
        """ Angle of axis 6 """
        return self.z

    @property
    def rail(self):
        """ Location of external slide rail module """
        return self.d


@dataclass
class MirobotCartesians(featured_dataclass):
    """ A dataclass to hold Mirobot's cartesian values and roll/pitch/yaw angles. """
    x: float = None
    """ Position on X-axis """
    y: float = None
    """ Position of Y-axis """
    z: float = None
    """ Position of Z-axis """
    a: float = None
    """ Position of Roll angle """
    b: float = None
    """ Position of Pitch angle """
    c: float = None
    """ Position of Yaw angle """

    @property
    def tx(self):
        """ Position on X-axis """
        return self.x

    @property
    def ty(self):
        """ Position on Y-axis """
        return self.y

    @property
    def tz(self):
        """ Position on Z-axis """
        return self.z

    @property
    def rx(self):
        """ Position of Roll angle """
        return self.a

    @property
    def ry(self):
        """ Position of Pitch angle """
        return self.b

    @property
    def rz(self):
        """ Position of Yaw angle """
        return self.c


@dataclass
class MirobotStatus(basic_dataclass):
    """ A composite dataclass to hold all of Mirobot's trackable quantities. """
    state: str = ''
    """ The brief descriptor string for Mirobot's state. """
    angle: MirobotAngles = MirobotAngles()
    """ Dataclass that holds Mirobot's angular values including the rail position value. """
    cartesian: MirobotCartesians = MirobotCartesians()
    """ Dataclass that holds the cartesian values and roll/pitch/yaw angles. """
    pump_pwm: int = None
    """ The current pwm of the pnuematic pump module. """
    valve_pwm: int = None
    """ The current pwm of the value module. (eg. gripper) """
    motion_mode: bool = False
    """ Whether Mirobot is currently in coordinate mode (`False`) or joint-motion mode (`True`) """



from .mirobot import Mirobot
from .base_mirobot import BaseMirobot
from .mirobot_status import MirobotStatus, MirobotAngles, MirobotCartesians

# don't document our resources directory duh
__pdoc__ = {}
__pdoc__['resources'] = False
__pdoc__['resources.__init__'] = False


# if someone imports by '*' then import everything in the following modules
__all__ = ['mirobot', 'base_mirobot', 'mirobot_status']



import logging


class ExitOnExceptionStreamHandler(logging.StreamHandler):
    def emit(self, record):
        super().emit(record)
        if record.levelno >= logging.ERROR:
            raise SystemExit(-1)


class MirobotError(Exception):
    """ An inplace class for throwing Mirobot errors. """
    pass


class MirobotAlarm(Exception):
    """  An inplace class for throwing Mirobot alarms. """
    pass


class MirobotReset(Exception):
    """ An inplace class for when Mirobot resets. """
    pass


class MirobotAmbiguousPort(Exception):
    """ An inplace class for when the serial port is unconfigurable. """
    pass


class MirobotStatusError(Exception):
    """ An inplace class for when Mirobot's status message is unprocessable. """
    pass


class MirobotResetFileError(Exception):
    """ An inplace class for when Mirobot has problems using the given reset file. """
    pass


class MirobotVariableCommandError(Exception):
    """ An inplace class for when Mirobot finds a command that does not match variable setting-command syntax. """
    pass


class SerialDeviceReadError(Exception):
    """ An inplace class for when SerialDevice is unable to read the serial port """


class SerialDeviceOpenError(Exception):
    """ An inplace class for when SerialDevice is unable to open the serial port """


class SerialDeviceLockError(Exception):
    """ An inplace class for when SerialDevice is unable to lock the serial port """


class SerialDeviceCloseError(Exception):
    """ An inplace class for when SerialDevice is unable to close the serial port """


class SerialDeviceUnlockError(Exception):
    """ An inplace class for when SerialDevice is unable to unlock the serial port """


class SerialDeviceWriteError(Exception):
    """ An inplace class for when SerialDevice is unable to write to the serial port """
